
#include<sstream>
#include "planDisplay.hh"
#include<boost/lexical_cast.hpp>

using namespace std;

planDisplay::planDisplay(const hatpPlan& currentPlan):
	currentPlan(currentPlan)
{
	vector<string> agents (currentPlan.getAgentsList());
	unsigned int nbAssignedColors;
	for(unsigned int l = 0 ;  l < agents.size() ; ++l){
		if( agentsToRankInColors.find(agents[l]) == agentsToRankInColors.end() ) {
			nbAssignedColors = agentsToRankInColors.size();
			agentsToRankInColors[agents[l]] = nbAssignedColors % NB_AGENTS_COLORS;
		}
	}
}
planDisplay::~planDisplay(){}

vector<float> parseNumberList(char* list) {
	vector<float> vec; string s(list);
	unsigned int start=0;
	for(unsigned int i=0 ; i<s.size() ; ++i) {
		if(s[i] == ',') {
			try { vec.push_back(boost::lexical_cast<float>(s.substr(start,i-start))); }
			catch(boost::bad_lexical_cast &) {}
			start = i+1;
		}
		else if(i == s.size()-1) {
			try { vec.push_back(boost::lexical_cast<float>(s.substr(start,i-start+1))); }
			catch(boost::bad_lexical_cast &) {}
		}
	}
	return vec;
}

string planDisplay::createStreamImage() {
	GVC_t* gvc = gvContext();
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t *subgraph, *graph = agopen((char *)"graph",AGDIGRAPHSTRICT);
	graph = agopen((char *)"graphForClustersContents", AGDIGRAPHSTRICT);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t *subgraph, *graph = agopen((char *)"graph",Agstrictdirected,NULL);
	graph = agopen((char *)"graphForClustersContents",Agstrictdirected,NULL);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
	agsafeset(graph, (char *)"bgcolor", (char *)"white", (char *)"");
	agsafeset(graph, (char *)"clusterrank", (char *)"local", (char *)"");
	agsafeset(graph, (char *)"rankdir", (char *)"LR", (char *)"TB");
	agsafeset(graph, (char *)"splines", (char *)"true", (char *)"false");
	ostringstream tmp;
	map<unsigned int, Agnode_t*> gphzNodes;
	int red, green, blue;
	vector<string> agents;
	ostringstream tmpHex;
	for(unsigned int i = 0 ; i <= currentPlan.getStreams()->getMaxRank() ; i++) {
		map<string, unsigned int> nodesWithSameRank;
		map<string, unsigned int>::iterator nwsrit;
		tmp.str(""); tmp << "sub" << i;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		subgraph = agsubg(graph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		subgraph = agsubg(graph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		agsafeset(subgraph, (char *)"rank", (char *)"same", (char *)"");
		vector<hatpNode*> tempnrank = currentPlan.getStreams()->getNodesOfRank(i);
		for(unsigned int j = 0 ; j < tempnrank.size() ; j++) {
			nodesWithSameRank[tempnrank[j]->getAgents()[0]] = tempnrank[j]->getID();
		}
		for(nwsrit = nodesWithSameRank.begin(); nwsrit != nodesWithSameRank.end(); nwsrit++) {
			tmp.str(""); tmp << "node" << nwsrit->second;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
			gphzNodes[nwsrit->second] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
			gphzNodes[nwsrit->second] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			ostringstream actionLabel;
			actionLabel.clear();
			actionLabel << currentPlan.getNode(nwsrit->second)->getName() << "\n(";
			vector<string> parameters = currentPlan.getNode(nwsrit->second)->getParameters();
			for(unsigned int k = 0 ; k < parameters.size() ; k++) actionLabel << parameters[k] << " ";
			actionLabel << ")\n";
			actionLabel << "["<< currentPlan.getNode(nwsrit->second)->getStartTime() << ", ";
			actionLabel << currentPlan.getNode(nwsrit->second)->getEndTime() << "]";
			string nodeLabel = actionLabel.str();
			agsafeset(gphzNodes[nwsrit->second], (char *)"label", const_cast<char*>(nodeLabel.c_str()), (char *)"");
			agsafeset(gphzNodes[nwsrit->second], (char *)"style", (char *)"filled", (char *)"");
				agents = currentPlan.getNode(nwsrit->second)->getAgents();
				if(agents.size() == 1) {
					tmpHex.str("");
					tmpHex << "#" << hex << colorsRED[agentsToRankInColors[agents[0]]];
					tmpHex << colorsGREEN[agentsToRankInColors[agents[0]]];
					tmpHex << colorsBLUE[agentsToRankInColors[agents[0]]];
				}
				else {
					tmpHex.str("");
					tmpHex << "#" << hex;
					red = 0; green = 0; blue = 0;
					for(unsigned int l2 = 0 ; l2 < agents.size() ; l2++) {
						red += colorsRED[agentsToRankInColors[agents[l2]]];
						green += colorsGREEN[agentsToRankInColors[agents[l2]]];
						blue += colorsBLUE[agentsToRankInColors[agents[l2]]];
					}
					tmpHex << red/agents.size() << green/agents.size() << blue/agents.size();
				}
				agsafeset(gphzNodes[nwsrit->second], (char *)"fillcolor", const_cast<char*>(tmpHex.str().c_str()), (char *)"");
			if( (currentPlan.getNode(nwsrit->second)->getAgents()).size() > 1 )
				agsafeset(gphzNodes[nwsrit->second], (char *)"peripheries", (char *)"2", (char *)"1");
		}
	}
	Agedge_t* edge;
	vector<unsigned int>succID;
	vector<hatpNode*> succ, TLFDnodes;
	TLFDnodes = currentPlan.getStreams()->getNodes();
	for(unsigned int m = 0; m < TLFDnodes.size() ; m++) {
		succID = TLFDnodes[m]->getStreamSuccessors();
		succ.clear();
		for(unsigned int o = 0 ; o < succID.size() ; o++) succ.push_back(currentPlan.getNode(succID[o]));
		for(unsigned int n = 0 ;  n <succ.size() ; n++) {
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
			edge = agedge(graph, gphzNodes[TLFDnodes[m]->getID()], gphzNodes[succ[n]->getID()]);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
			edge = agedge(graph, gphzNodes[TLFDnodes[m]->getID()], gphzNodes[succ[n]->getID()], (char *)"", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			agsafeset(edge, (char *)"color", (char *)"darkorchid3", (char *)"black");
			agsafeset(edge, (char *)"arrowhead", (char *)"open", (char *)"normal");
		}
	}
	gvLayout(gvc, graph, "dot");
	string fileName("planStreams.svg");
	gvRenderFilename(gvc, graph, "svg", const_cast<char*>(fileName.c_str()) );
	gvFreeLayout(gvc, graph);
	agclose(graph);
	gvFreeContext(gvc);
	return fileName;
}





string planDisplay::createTreeImage() {
	GVC_t* gvc = gvContext();
	Agraph_t *graphForClustersContents, *cluster, *subgraph;
	const float POINTS_BY_INCH = 72.0;
	map<string, pair<float,float> > clustersDimensionsInInches;
	map<unsigned int, pair<float,float> > nodesDimensionsInInches, nodesCenterRelPosInInches;
	map<unsigned int, string> nodesToClusters;
	vector<hatpTaskList*> PTnodes = currentPlan.getTree()->getTreeAsTaskLists();
	hatpTaskList* PTTLFD;
	vector<unsigned int> PTTLFDnodes, succ;
	map<unsigned int, Agnode_t*> gphzNodes;
	map<unsigned int, Agnode_t*>::const_iterator ITgphzNodes;
	vector<float> numericData, numericData2;
	ostringstream tmp;
	for(unsigned int i = 0 ; i < PTnodes.size() ; i++) {
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		graphForClustersContents = agopen((char *)"graphForClustersContents", AGDIGRAPHSTRICT);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		graphForClustersContents = agopen((char *)"graphForClustersContents", Agstrictdirected, NULL);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		agsafeset(graphForClustersContents, (char *)"compound", (char *)"true", (char *)"false");
		agsafeset(graphForClustersContents, (char *)"bgcolor", (char *)"white", (char *)"");
		agsafeset(graphForClustersContents, (char *)"clusterrank", (char *)"local", (char *)"");
		agsafeset(graphForClustersContents, (char *)"rankdir", (char *)"LR", (char *)"TB");
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		cluster = agsubg(graphForClustersContents, (char *)"cluster0");
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		cluster = agsubg(graphForClustersContents, (char *)"cluster0", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		gphzNodes.clear();
		PTTLFD = PTnodes[i];
		for(unsigned int j = 0 ; j <= PTTLFD->getMaxRank() ; j++) {
			vector<unsigned int>  nodesWithSameRank;
			tmp.str(""); tmp << "sub" << j ;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
			subgraph = agsubg(cluster, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
			subgraph = agsubg(cluster, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			agsafeset(subgraph, (char *)"rank", (char *)"same", (char *)"");
			vector<hatpNode*> tempnrank = PTTLFD->getNodesOfRank(j);
			for(unsigned int k = 0 ; k < tempnrank.size() ; k++) nodesWithSameRank.push_back(tempnrank[k]->getID());
			for(unsigned int k = 0 ; k < nodesWithSameRank.size() ; k++) {
				tmp.str(""); tmp << "node" << nodesWithSameRank[k];
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
				gphzNodes[nodesWithSameRank[k]] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
				gphzNodes[nodesWithSameRank[k]] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
				ostringstream actionLabel;
				actionLabel.clear();
				actionLabel << currentPlan.getNode(nodesWithSameRank[k])->getName() << "\n(";
				vector<string> parameters = currentPlan.getNode(nodesWithSameRank[k])->getParameters();
				for(unsigned int l = 0 ; l < parameters.size() ; l++) actionLabel << parameters[l] << " ";
				actionLabel << ")";
				string nodeLabel = actionLabel.str();
				agsafeset(gphzNodes[nodesWithSameRank[k]], (char *)"label", const_cast<char*>(nodeLabel.c_str()), (char *)"ERROR");
			}
		}
		// we complete the graph with edges (causal links) of the current Plan Tree node
		map<unsigned int, hatpNode*> ttnodes = PTTLFD->getNodes();
		PTTLFDnodes.clear();
		for(map<unsigned int, hatpNode*>::iterator it = ttnodes.begin() ; it != ttnodes.end() ; it++) {
			PTTLFDnodes.push_back((*it).first);
		}
		for(unsigned int j = 0 ; j < PTTLFDnodes.size() ; j++) {
			succ = currentPlan.getNode(PTTLFDnodes[j])->getTreeSuccessors();
			for(unsigned int k = 0 ; k < succ.size() ; k++)
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
				agedge(cluster, gphzNodes[PTTLFDnodes[j]], gphzNodes[succ[k]]);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
				agedge(cluster, gphzNodes[PTTLFDnodes[j]], gphzNodes[succ[k]], (char *)"", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		}
		gvLayout(gvc, graphForClustersContents, "dot");
		gvRender(gvc, graphForClustersContents, "dot", NULL);
		// storage of cluster dimensions
		numericData = parseNumberList(agget(cluster, (char *)"bb"));
		clustersDimensionsInInches[PTnodes[i]->toString()] = make_pair((numericData[2]-numericData[0])/POINTS_BY_INCH,
		                                                   (numericData[3]-numericData[1])/POINTS_BY_INCH);
		for(ITgphzNodes = gphzNodes.begin() ; ITgphzNodes != gphzNodes.end() ; ITgphzNodes++) {
			numericData = parseNumberList(agget(ITgphzNodes->second,(char *)"width"));
			numericData2 = parseNumberList(agget(ITgphzNodes->second,(char *)"height"));
			nodesDimensionsInInches[ITgphzNodes->first] = make_pair( numericData[0], numericData2[0] );
			numericData = parseNumberList(agget(cluster,(char *)"bb"));
			numericData2 = parseNumberList(agget(ITgphzNodes->second,(char *)"pos"));
			nodesCenterRelPosInInches[ITgphzNodes->first]
				= make_pair( (numericData2[0] - (numericData[0] + numericData[2])/2.0)/POINTS_BY_INCH,
				            (numericData2[1] - (numericData[1] + numericData[3])/2.0)/POINTS_BY_INCH );
			nodesToClusters[ITgphzNodes->first] = PTnodes[i]->toString();

		}
		gvFreeLayout(gvc, graphForClustersContents);
		agclose(graphForClustersContents);
	}
	// we create the graph that will be used to determine cluster scheduling in a TOP BOTTOM kind
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t* graph = agopen((char *)"graph", AGDIGRAPHSTRICT);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t* graph = agopen((char *)"graph", Agstrictdirected, NULL);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
	agsafeset(graph, (char *)"bgcolor", (char *)"white", (char *)"");
	agsafeset(graph, (char *)"rankdir", (char *)"TB", (char *)"TB");
	// we create this graph by level in the tree, we start by creating node
	map<string, Agnode_t*> gphzPTnodes;
	map<string, Agnode_t*>::const_iterator ITgphzPTnodes;
	ostringstream tmp2;
	for(unsigned int i=0 ; i <= currentPlan.getTree()->getTreeDepth() ; i++) {
		PTnodes = currentPlan.getTree()->getTaskListsAtLevel(i);
		// we create a subgraph to ensure node ranking
		tmp.str(""); tmp << "level" << i;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		subgraph = agsubg(graph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		subgraph = agsubg(graph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		agsafeset(subgraph, (char *)"rank", (char *)"same", (char *)"");
		// we add a node for each Plan Tree Node with same dimensions as determined before
		for(unsigned int j=0 ; j<PTnodes.size() ; j++) {
			tmp.str(""); tmp << "level" << i << "_" << j;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
			gphzPTnodes[PTnodes[j]->toString()] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
			gphzPTnodes[PTnodes[j]->toString()] = agnode(subgraph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			agsafeset(gphzPTnodes[PTnodes[j]->toString()], (char *)"shape", (char *)"box", (char *)"ellipse");
			agsafeset(gphzPTnodes[PTnodes[j]->toString()], (char *)"label", (char *)" ", (char *)"");
			tmp2.str(""); tmp2 << clustersDimensionsInInches[PTnodes[j]->toString()].first;
			agsafeset(gphzPTnodes[PTnodes[j]->toString()], (char *)"width", const_cast<char*>(tmp2.str().c_str()), (char *)"0.75");
			tmp2.str(""); tmp2 << clustersDimensionsInInches[PTnodes[j]->toString()].second;
			agsafeset(gphzPTnodes[PTnodes[j]->toString()], (char *)"height", const_cast<char*>(tmp2.str().c_str()), (char *)"0.5");
		}
	}
	// we add the edges (hierarchical links)
	PTnodes = currentPlan.getTree()->getTreeAsTaskLists();
	for(unsigned int i = 0 ; i < PTnodes.size() ;i++) {
		for(unsigned int j = i+1 ; j < PTnodes.size() ; j++) {
			if(PTnodes[i]->isTaskListDecomposition(PTnodes[j])) {
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
				agedge(graph, gphzPTnodes[PTnodes[i]->toString()], gphzPTnodes[PTnodes[j]->toString()]);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
				agedge(graph, gphzPTnodes[PTnodes[i]->toString()], gphzPTnodes[PTnodes[j]->toString()], (char *)"", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			}
		}
	}
	gvLayout(gvc, graph, "dot");
	gvRender(gvc, graph, "dot", NULL);
	gvFreeLayout(gvc, graph);
	// we create the final graph and we make a copy of all nodes of graph
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t* finalGraph = agopen((char *)"final_graph", AGDIGRAPHSTRICT);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
	Agraph_t* finalGraph = agopen((char *)"final_graph", Agstrictdirected, NULL);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
	agsafeset(finalGraph, (char *)"overlap", (char *)"true", (char *)"false"); // to allow nodes overlapping
	agsafeset(finalGraph, (char *)"outputorder", (char *)"nodesfirst", (char *)"breadthfirst"); // to draw nodes first
	map<string, Agnode_t*> finalGraphclusters;
	int nb=0;
	for(ITgphzPTnodes = gphzPTnodes.begin() ; ITgphzPTnodes != gphzPTnodes.end() ; ITgphzPTnodes++) {
		tmp.str("");
		tmp << "cluster_" << nb++;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		finalGraphclusters[ITgphzPTnodes->first] = agnode(finalGraph, const_cast<char*>(tmp.str().c_str()) );
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		finalGraphclusters[ITgphzPTnodes->first] = agnode(finalGraph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"shape", agget(ITgphzPTnodes->second, (char *)"shape"), (char *)"ellipse");
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"label", agget(ITgphzPTnodes->second, (char *)"label"), (char *)"");
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"width", agget(ITgphzPTnodes->second, (char *)"width"), (char *)"");
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"height", agget(ITgphzPTnodes->second, (char *)"height"), (char *)"");
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"pin", (char *)"true", (char *)"false");
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"color", (char *)"blue", (char *)"black");
		numericData = parseNumberList(agget(ITgphzPTnodes->second, (char *)"pos"));
		tmp.str("");
		tmp << numericData[0]/POINTS_BY_INCH << "," << numericData[1]/POINTS_BY_INCH;
		agsafeset(finalGraphclusters[ITgphzPTnodes->first],(char *)"pos", const_cast<char*>( tmp.str().c_str() ), (char *)"");
	}
	agclose(graph);
	int red, green, blue;
	vector<string> agents;
	ostringstream tmpHex;
	// we complete the computed layout with nodes produced during clusters scheduling phase
	map<unsigned int, pair<float,float> >::const_iterator itPTTLnodes;
	gphzNodes.clear();
	for(itPTTLnodes = nodesCenterRelPosInInches.begin(); itPTTLnodes != nodesCenterRelPosInInches.end() ; itPTTLnodes++) {
		tmp.str(""); tmp << "node_" << itPTTLnodes->first;
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
		gphzNodes[itPTTLnodes->first] = agnode(finalGraph, const_cast<char*>(tmp.str().c_str()));
#else //GRAPHVIZ_AGSUBG_2_PARAMS
		gphzNodes[itPTTLnodes->first] = agnode(finalGraph, const_cast<char*>(tmp.str().c_str()), true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"shape", (char *)"ellipse", (char *)"ellipse"); // shape = ellipse
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"pin", (char *)"true", (char *)"false"); // we freeze their positions
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"style", (char *)"filled", (char *)""); // to allow nodes with colors
		// we give them the good dimensions
		tmp2.str(""); tmp2 << nodesDimensionsInInches[itPTTLnodes->first].first;
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"width", const_cast<char*>(tmp2.str().c_str()), (char *)"0.75");
		tmp2.str(""); tmp2 << nodesDimensionsInInches[itPTTLnodes->first].second;
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"height", const_cast<char*>(tmp2.str().c_str()), (char *)"0.5");
		// we give them their label
		ostringstream actionLabel;
		actionLabel.clear();
		actionLabel << currentPlan.getNode(itPTTLnodes->first)->getName() << "\n(";
		vector<string> parameters = currentPlan.getNode(itPTTLnodes->first)->getParameters();
		for(unsigned int k = 0 ; k < parameters.size() ; k++) actionLabel << parameters[k] << " ";
		actionLabel << ")";
		string nodeLabel = actionLabel.str();
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"label", const_cast<char*>( nodeLabel.c_str() ), (char *)"");
		// we give them their position
		numericData = parseNumberList( agget(finalGraphclusters[nodesToClusters[itPTTLnodes->first]], (char *)"pos") );
		tmp2.str("");
		tmp2 << numericData[0] + (itPTTLnodes->second).first << ","; // pos contains values in inches
		tmp2 << numericData[1] + (itPTTLnodes->second).second;
		agsafeset(gphzNodes[itPTTLnodes->first], (char *)"pos", const_cast<char*>(tmp2.str().c_str()), (char *)"");
		// we give them a color depending on it is a task/action and involved agents
		if( !(currentPlan.getNode(itPTTLnodes->first)->isAction()))
			agsafeset(gphzNodes[itPTTLnodes->first], (char *)"fillcolor", (char *)"azure3", (char *)"");
		else { // it is an action (colors is always apply on actions) or it is a task and we apply special render
			agents = currentPlan.getNode(itPTTLnodes->first)->getAgents();
			// we compute the color
			if(agents.size() == 1) { // only one agent involved, we apply its color
				tmpHex.str(""); tmpHex << "#" << hex << colorsRED[agentsToRankInColors[agents[0]]];
				tmpHex << colorsGREEN[agentsToRankInColors[agents[0]]] << colorsBLUE[agentsToRankInColors[agents[0]]];
			}
			else { // several agents involved, we mix their colors
				tmpHex.str(""); tmpHex << "#" << hex;
				red = 0; green = 0; blue = 0;
				for(unsigned int l = 0 ; l < agents.size() ; l++) {
					red += colorsRED[agentsToRankInColors[agents[l]]];
					green += colorsGREEN[agentsToRankInColors[agents[l]]];
					blue += colorsBLUE[agentsToRankInColors[agents[l]]];
				}
				tmpHex << red/agents.size() << green/agents.size() << blue/agents.size();
			}
			agsafeset(gphzNodes[itPTTLnodes->first], (char *)"fillcolor", const_cast<char*>(tmpHex.str().c_str()), (char *)"");
		}
		if(currentPlan.getNode(itPTTLnodes->first)->getAgents().size() > 1)
			agsafeset(gphzNodes[itPTTLnodes->first], (char *)"peripheries", (char *)"2", (char *)"1");
	}
	// we add edges associated to causal and hierarchical links
	Agedge_t* edge;
	vector<hatpNode*> PTTLnodes;
	vector<unsigned int> tsucc;
	PTTLnodes = currentPlan.getNodes();
	for(unsigned int i = 0 ; i < PTTLnodes.size() ; i++) {
		succ = PTTLnodes[i]->getTreeSuccessors();
		for(unsigned int k=0 ; k<succ.size() ; ++k) { // causal links
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
			edge = agedge(finalGraph, gphzNodes[PTTLnodes[i]->getID()], gphzNodes[succ[k]]);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
			edge = agedge(finalGraph, gphzNodes[PTTLnodes[i]->getID()], gphzNodes[succ[k]], (char *)"", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
			agsafeset(edge, (char *)"color", (char *)"darkorchid3", (char *)"black");
			agsafeset(edge, (char *)"arrowhead", (char *)"open", (char *)"normal");
		}
	}
	PTnodes = currentPlan.getTree()->getTreeAsTaskLists();
	for(unsigned int i = 0 ; i < PTnodes.size() ;i++) {
		for(unsigned int j = i+1 ; j < PTnodes.size() ; j++) {
			if(PTnodes[i]->isTaskListDecomposition(PTnodes[j])) {
				unsigned int rootn = (PTnodes[i]->getRootTaskListDecomposition(PTnodes[j]))->getID();
#ifdef GRAPHVIZ_AGSUBG_2_PARAMS
				edge = agedge(finalGraph, gphzNodes[rootn], finalGraphclusters[PTnodes[j]->toString()]);
#else //GRAPHVIZ_AGSUBG_2_PARAMS
				edge = agedge(finalGraph, gphzNodes[rootn], finalGraphclusters[PTnodes[j]->toString()], (char *)"", true);
#endif //GRAPHVIZ_AGSUBG_2_PARAMS
				agsafeset(edge, (char *)"style", (char *)"bold", (char *)"");
			}
		}
	}
	gvLayout(gvc, finalGraph, "fdp");
	string fileName("planTree.svg");
	gvRenderFilename(gvc, finalGraph, "svg", const_cast<char*>(fileName.c_str()) );
	gvFreeLayout(gvc, finalGraph);
	agclose(finalGraph);
	gvFreeContext(gvc);
	return fileName;
}


